- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository:
        metadata:
          format: lava-test test definition 1.0
          name: command
          description: "Run command"
        run:
          steps:
          - "lava-test-case command --shell {{ command }}"
      from: inline
      path: command
      name: command
